# Norman encoding for mkt1font and vpl2vpl
#
# Encoding devised by Prof. K. R. Norman for the representation of
# Indian languages in Roman script

# Chars 0-31

0	.notdef
1	Eth
2	eth
3	Lslash
4	lslash
5	S caron
6	s caron
7	Y acute
8	y acute
9	.notdef
10	.notdef
11	Thorn
12	thorn
13	.notdef
14	Z caron
15	z caron
16	.notdef
17	asciitilde
18	j macron			# j is dotless in Normyn
19	circumflex
20	tilde
21	onehalf
22	onequarter
23	onesuperior
24	threequarters
25	threesuperior
26	twosuperior
27	brokenbar
28	minus
29	multiply
30	.notdef
31	.notdef

# Changes from ASCII in range 32-127

39	quotesingle
94	circumflex
96	grave

# Chars 128-255

128	A dieresis
129	A ring
130	imacron tilde
131	E acute
132	N tilde
133	O dieresis
134	U dieresis
135	a acute
136	a grave
137	a circumflex
138	a dieresis
139	a tilde
140	a macron
141	c cedilla
142	e acute
143	e grave
144	e circumflex
145	e dieresis
146	i acute
147	i grave
148	i circumflex
149	i dieresis
150	n tilde
151	o acute
152	o grave
153	o circumflex
154	o dieresis
155	o tilde
156	u acute
157	u grave
158	u circumflex
159	u dieresis
160	t underdot			# see also 173, 202
161	degree
162	e tilde
163	sterling
164	section
165	runderdot macron		# runderring macron in Normyn
166	paragraph
167	s acute
168	r underdot
169	s caron
170	umacron breve
171	acute
172	dieresis
173	t underdot			# notequal in Normyn
174	AE
175	o macron
176	i tilde
177	plusminus
178	amacron acute
179	umacron acute
180	i macron
181	m underdot
182	d underdot
183	s underdot
184	emacron acute
185	z acute
186	n overdot
187	emacron breve
188	omacron breve
189	u tilde
190	ae
191	oslash
192	macron breve
193	amacron breve
194	l underdot
195	j macron			# j is dotless in Normyn
196	a breve
197	u breve
198	amacron tilde
199	imacron acute
200	M underdot
201	ellipsis
202	.notdef				# t underdot in PC Normyn
203	A grave
204	A tilde
205	O tilde
206	m breve
207	oe
208	endash
209	emdash
210	quotedblleft
211	quotedblright
212	quoteleft
213	quoteright
214	l underring
215	m overdot
216	n breve
217	multiply
218	runderdotmacron acute		# runderringmacron acute in Normyn
219	r underring
220	N overdot
221	S underdot
222	H underdot
223	D underdot
224	daggerdbl
225	runderdot acute			# runderring acute in Normyn
226	L underdot
227	e macron
228	imacron breve
229	r underbar
230	r underdieresis
231	I macron
232	u macron
233	T underdot
234	S acute
235	dagger
236	e breve
237	o breve
238	A macron
239	N underdot
240	umacron tilde
241	l underbar
242	U macron
243	Z acute
244	R underdot			# R underring in Normyn
245	dotlessi
246	n underdot
247	n underbar
248	macron
249	breve
250	h underdot
251	i breve
252	cedilla
253	hungarumlaut
254	ogonek
255	caron
