# ISO-Latin1 encoding (iso8859-1) for mkt1font and vpl2vpl

127	.notdef
128	.notdef
129	.notdef
130	.notdef
131	.notdef
132	.notdef
133	.notdef
134	.notdef
135	.notdef
136	.notdef
137	.notdef
138	.notdef
139	.notdef
140	.notdef
141	.notdef
142	.notdef
143	.notdef
144	dotlessi
145	grave
146	acute
147	circumflex
148	tilde
149	.notdef
150	breve
151	dotaccent
152	.notdef
153	.notdef
154	ring
155	.notdef
156	.notdef
157	hungarumlaut
158	ogonek
159	caron
160	.notdef
161	exclamdown
162	cent
163	sterling
164	currency
165	yen
166	brokenbar
167	section
168	dieresis
169	copyright
170	ordfeminine
171	guillemotleft
172	logicalnot
173	hyphen
174	registered
175	macron
176	degree
177	plusminus
178	twosuperior
179	threesuperior
180	acute
181	mu
182	paragraph
183	periodcentered
184	cedilla
185	onesuperior
186	ordmasculine
187	guillemotright
188	onequarter
189	onehalf
190	threequarters
191	questiondown
192	A grave
193	A acute
194	A circumflex
195	A tilde
196	A dieresis
197	A ring
198	AE
199	C cedilla
200	E grave
201	E acute
202	E circumflex
203	E dieresis
204	I grave
205	I acute
206	I circumflex
207	I dieresis
208	Eth
209	N tilde
210	O grave
211	O acute
212	O circumflex
213	O tilde
214	O dieresis
215	multiply
216	Oslash
217	U grave
218	U acute
219	U circumflex
220	U dieresis
221	Y acute
222	Thorn
223	germandbls
224	a grave
225	a acute
226	a circumflex
227	a tilde
228	a dieresis
229	a ring
230	ae
231	c cedilla
232	e grave
233	e acute
234	e circumflex
235	e dieresis
236	i grave
237	i acute
238	i circumflex
239	i dieresis
240	eth
241	n tilde
242	o grave
243	o acute
244	o circumflex
245	o tilde
246	o dieresis
247	divide
248	oslash
249	u grave
250	u acute
251	u circumflex
252	u dieresis
253	y acute
254	thorn
255	y dieresis
