# IndUni encoding for mkt1font and vpl2vpl
#
# This version designed for use with Omega
#
# Characters are intended to represent a complete
# set of those required for use with Indian languages in
# transliteration, together with others commonly needed
# in the same environment. The "non-Indian" characters 
# form an almost complete implementation on MES1, but
# are limited by the fact that some glyphs, though
# present in later versions of the URW++ fonts, are not
# present in all, and anyway cannot readily be accessed.
# Such items are included in this list but commented out.

# ASCII: I should probably put in a complete list, or at least
# notdef the control codes and return other odds and ends to
# ASCII normality. I would need to know Omega policies here.

0x007f	.notdef

# ISO-Latin1

0x0080	.notdef
0x0081	.notdef
0x0082	.notdef
0x0083	.notdef
0x0084	.notdef
0x0085	.notdef
0x0086	.notdef
0x0087	.notdef
0x0088	.notdef
0x0089	.notdef
0x008a	.notdef
0x008b	.notdef
0x008c	.notdef
0x008d	.notdef
0x008e	.notdef
0x008f	.notdef
0x0090	.notdef
0x0091	.notdef
0x0092	.notdef
0x0093	.notdef
0x0094	.notdef
0x0095	.notdef
0x0096	.notdef
0x0097	.notdef
0x0098	.notdef
0x0099	.notdef
0x009a	.notdef
0x009b	.notdef
0x009c	.notdef
0x009d	.notdef
0x009e	.notdef
0x009f	.notdef
0x00a0	space
0x00a1	exclamdown
0x00a2	cent
0x00a3	sterling
0x00a4	currency
0x00a5	yen
#0x00a6	brokenbar
0x00a7	section
0x00a8	dieresis
#0x00a9	copyright
0x00aa	ordfeminine
0x00ab	guillemotleft
#0x00ac	logicalnot
0x00ad	hyphen
#0x00ae	registered
0x00af	macron
#0x00b0	degree
#0x00b1	plusminus
#0x00b2	twosuperior
#0x00b3	threesuperior
0x00b4	acute
#0x00b5	mu
0x00b6	paragraph
0x00b7	periodcentered
0x00b8	cedilla
#0x00b9	onesuperior
0x00ba	ordmasculine
0x00bb	guillemotright
#0x00bc	onequarter
#0x00bd	onehalf
#0x00be	threequarters
0x00bf	questiondown
0x00c0	A grave
0x00c1	A acute
0x00c2	A circumflex
0x00c3	A tilde
0x00c4	A dieresis
0x00c5	A ring
0x00c6	AE
0x00c7	C cedilla
0x00c8	E grave
0x00c9	E acute
0x00ca	E circumflex
0x00cb	E dieresis
0x00cc	I grave
0x00cd	I acute
0x00ce	I circumflex
0x00cf	I dieresis
#0x00d0	Eth
0x00d1	N tilde
0x00d2	O grave
0x00d3	O acute
0x00d4	O circumflex
0x00d5	O tilde
0x00d6	O dieresis
#0x00d7	multiply
0x00d8	Oslash
0x00d9	U grave
0x00da	U acute
0x00db	U circumflex
0x00dc	U dieresis
0x00dd	Y acute
#0x00de	Thorn
0x00df	germandbls
0x00e0	a grave
0x00e1	a acute
0x00e2	a circumflex
0x00e3	a tilde
0x00e4	a dieresis
0x00e5	a ring
0x00e6	ae
0x00e7	c cedilla
0x00e8	e grave
0x00e9	e acute
0x00ea	e circumflex
0x00eb	e dieresis
0x00ec	i grave
0x00ed	i acute
0x00ee	i circumflex
0x00ef	i dieresis
#0x00f0	eth
0x00f1	n tilde
0x00f2	o grave
0x00f3	o acute
0x00f4	o circumflex
0x00f5	o tilde
0x00f6	o dieresis
#0x00f7	divide
0x00f8	oslash
0x00f9	u grave
0x00fa	u acute
0x00fb	u circumflex
0x00fc	u dieresis
0x00fd	y acute
#0x00fe	thorn
0x00ff	y dieresis

# Bare accents.

0x0327	cedilla
0x0328	ogonek
#0x0326	commaaccent
0x0301	acute
0x0300	grave
0x0302	circumflex
0x0303	tilde
0x0304	macron
0x0306	breve
0x0307	space overdot
0x0308	dieresis
0x030a	ring
0x030b	hungarumlaut
0x030c	caron
0x0310	space candrabindu
0x0331	space underbar
0x032e	space underbreve
0x0324	space underdieresis
0x0323	space underdot
0x0325	space underring

# Indian etc. characters

0x0102	A breve
0x01cd	A caron
0x0100	A macron
#0x0104	A ogonek
0x01fc	AE acute
0x01e2	AE macron
0xe000	Amacron tilde
0xe001	Atilde acute
0x0106	C acute
0x010c	C caron
0x0108	C circumflex
0xe002	C macron
0x010a	C overdot
0xe003	C underdot
#0x010E	D caron
0x1e0c	D underdot
0xe004	Dunderdot acute
#0x0110	Dcroat
0x0114	E breve
0x011a	E caron
0x0112	E macron
0x1ebc	E tilde
#0x0118	E ogonek
0x0116	E overdot
0xe006	Emacron tilde
0xe005	Etilde acute
0xe007	G acute
#0x0122	G cedilla
0x0120	G overdot
0xe008	Gacute macron
0xe009	H acute
0x0124	H circumflex
0xe00b	H underbar
0x1e2a	H underbreve
0xe00a	H underdieresis
0x1e24	H underdot
#0x0126	Hbar
0x012c	I breve
0x01cf	I caron
#0x012e	I ogonek
0x012a	I macron
0x0130	I overdot
0x0128	I tilde
0xe00d	Imacron tilde
0xe00c	Itilde acute
#0x0132	IJ
0x0134	J circumflex
0xe00f	J caron
0xe00e	J macron
0xe05e	K underdot
#0x0136	K cedilla
0xe012	K h
0xe010	K H
0x1e34	K underbar
0xe013	Kh underbar
0xe011	KH underbar
0x0139	L acute
#0x013d	L caron
#0x013b	L cedilla
0x1e3a	L underbar
0x1e36	L underdot
0xe014	L underring
#0x013f	L middot
0x0141	Lslash
0x1e38	Lunderdot macron
0xe015	Lunderring macron
0xe016	M breve
0xe017	M candrabindu
0x1e40	M overdot
0x1e42	M underdot
0x0143	N acute
0xe018	N breve
0x0147	N caron
0x1e44	N overdot
#0x0145	N cedilla
0x1e48	N underbar
0x1e46	N underdot
#0x014a	Eng
0x014e	O breve
0x01d1	O caron
0x014c	O macron
0x0150	O hungarumlaut
0x0152	OE
0xe019	Omacron tilde
0x1e4c	Otilde acute
0x1e54	P acute
0x0154	R acute
0xe01a	R breve
0x0158	R caron
#0x0156	R cedilla
0x1e5e	R underbar
0x1e5a	R underdot
0xe01b	R underring
0x1e5c	Runderdot macron
0xe01c	Runderring macron
0x015a	S acute
0x015c	S circumflex
0x0160	S caron
0x015e	S cedilla
0xe020	S underbar
0xe01f	S underdieresis
0x1e62	S underdot
0xe01d	Sacute macron
0xe01e	Sunderdot macron
#0x0164	T caron
0x0162	T cedilla
#0x0166	Tcroat
0x1e6e	T underbar
0xe022	T underdieresis
0x1e6c	T underdot
0xe021	Tunderdot acute
0x016c	U breve
0x01d3	U caron
0x016a	U macron
0x016e	U ring
0x0170	U hungarumlaut
#0x0172	U ogonek
0x0168	U tilde
0x01d7	Udieresis acute
0x01d9	Udieresis caron
0x01db	Udieresis grave
0x01d5	Udieresis macron
0xe023	Umacron tilde
0x1e78	Utilde acute
0xe024	V acute
0x0174	W circumflex
0x1e88	W underdot
0x0176	Y circumflex
0x0178	Y dieresis
0x1e8e	Y overdot
0x0179	Z acute
0x017d	Z caron
0x017b	Z overdot
0x1e94	Z underbar
0xe025	Z underdieresis
0x1e92	Z underdot
0x0103	a breve
0x01ce	a caron
0x0101	a macron
#0x0105	a ogonek
0x01fd	ae acute
0x01e3	ae macron
0xe026	amacron acute
0xe029	amacron breve
0xe027	amacron grave
0xe028	amacron tilde
0xe02a	atilde acute
0x0107	c acute
0x010d	c caron
0x0109	c circumflex
0xe02b	c macron
0x010b	c overdot
0xe02c	c underdot
#0x010f	d caron
#0x0111	dcroat
0x1e0d	d underdot
0xe02d	dunderdot acute
0x0115	e breve
0x011b	e caron
0x0113	e macron
0x1ebd	e tilde
0x0117	e overdot
#0x0119	e ogonek
0xe02f	emacron acute
0xe031	emacron breve
0xe030	emacron tilde
0xe02e	etilde acute
0xe032	g acute
0x0121	g overdot
#0x0123	g cedilla
0xe033	gacute macron
0xe034	h acute
0x0125	h circumflex
0x1e96	h underbar
0x1e2b	h underbreve
0xe035	h underdieresis
0x1e25	h underdot
#0x0127	hbar
0x012d	i breve
0x01d0	i caron
0x012b	i macron
0x0129	i tilde
#0x012f	i ogonek
0xe036	imacron acute
0xe039	imacron breve
0xe037	imacron grave
0xe038	imacron tilde
0xe03a	itilde acute
0x0131	dotlessi
#0x0133	ij
0x0135	j circumflex
0x01f0	j caron
0xe03b	j macron
#0x0137	k cedilla
0xe05f	k underdot
0xe03c	k h
0x1e35	k underbar
0xe03d	kh underbar
#0x0138	kra (Greenlandic)
0x013a	l acute
#0x013e	l caron
#0x013c	l cedilla
#0x0140	l middot
0xe03e	l tilde
0x1e3b	l underbar
0x1e37	l underdot
0xe040	l underring
0x0142	lslash
0xe03f	lunderdot acute
0x1e39	lunderdot macron
0xe041	lunderring acute
0xe042	lunderring macron
0xe043	m breve
0xe044	m candrabindu
0x1e41	m overdot
0x1e43	m underdot
0x0144	n acute
0x0148	n caron
#0x0146	n cedilla
0xe045	n breve
0x1e45	n overdot
0x1e49	n underbar
0x1e47	n underdot
#0x0149	n apostophe
#0x014b	eng
0x014f	o breve
0x01d2	o caron
0x014d	o macron
0x0151	o hungarumlaut
0x0153	oe
0xe047	omacron breve
0xe046	omacron tilde
0x1e4d	otilde acute
0x1e55	p acute
0x0155	r acute
0xe048	r breve
0x0159	r caron
#0x0156	r cedilla
0x1e5f	r underbar
0xe04c	r underdieresis
0x1e5b	r underdot
0xe04d	r underring
0xe049	runderdot acute
0xe04a	runderdot grave
0x1e5d	runderdot macron
0xe04b	runderdotmacron acute
0xe04e	runderring acute
0xe04f	runderring grave
0xe050	runderring macron
0xe051	runderringmacron acute
0x015b	s acute
0x015d	s circumflex
0x015f	s cedilla
0x0161	s caron
0xe055	s underbar
0xe054	s underdieresis
0x1e63	s underdot
0xe052	sacute macron
0xe053	sunderdot macron
#0x0165	t caron
0x0163	t cedilla
0x1e6f	t underbar
0xe057	t underdieresis
0x1e6d	t underdot
0xe056	tunderdot acute
#0x0167	tcroat
0x016d	u breve
0x01d4	u caron
0x016b	u macron
0x016f	u ring
0x0169	u tilde
0x0171	u hungarumlaut
#0x0173	u ogonek
0x01d8	udieresis acute
0x01da	udieresis caron
0x01dc	udieresis grave
0x01d6	udieresis macron
0xe058	umacron acute
0xe05b	umacron breve
0xe059	umacron grave
0xe05a	umacron tilde
0x1e79	utilde acute
0xe05c	v acute
0x0175	w circumflex
0x1e89	w underdot
0x0177	y circumflex
0x1e8f	y overdot
0x017a	z acute
0x017e	z caron
0x017c	z overdot
0x1e95	z underbar
0xe05d	z underdieresis
0x1e93	z underdot

# Odds and ends

0x2002	bullet
0x2020	dagger
0x2021	daggerdbl
0x2026	ellipsis
0x2014	emdash
0x2013	endash
0x0192	florin
0x2039	guilsinglleft
0x203a	guilsinglright
#0x2212	minus
0x2030	perthousand
0x201e	quotedblbase
0x201c	quotedblleft
0x201d	quotedblright
0x2018	quoteleft
0x2019	quoteright
0x201a	quotesinglbase
0x0027	quotesingle
#0x2122	trademark
